/*
    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    Ben Kibbey <bjk@luxsci.net>

    This file is part of pwmd.

    Pwmd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Pwmd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Pwmd.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef AGENT_H
#define AGENT_H

#include "pinentry.h"

struct inquire_data_s
{
  struct crypto_s *crypto;
  char *line;
  size_t len;
  int preset;
};

struct agent_s
{
  assuan_context_t ctx;
  assuan_context_t client_ctx;
  membuf_t data;
  gpg_error_t (*inquire_cb) (void *data, const char *line);
  void *inquire_data;
  void *inquire_data2;
  void *inquire_data3;
  struct pinentry_option_s pinentry_opts;
  int restart;
  int did_restart;
  size_t inquire_maxlen;
};

void cleanup_agent (struct agent_s *agent);
gpg_error_t send_to_agent (struct agent_s *agent, char **result, size_t * len,
			   const char *fmt, ...);
gpg_error_t agent_init (struct agent_s **);
gpg_error_t encrypt_data_file (assuan_context_t ctx, struct crypto_s *crypto,
			       gcry_sexp_t pubkey, gcry_sexp_t sigpkey,
			       const char *filename, const void *xml,
			       size_t len);

gpg_error_t generate_key (struct crypto_s *, char *, int, int);
gpg_error_t set_agent_option (struct agent_s *agent, const char *name,
			      const char *value);
gpg_error_t set_agent_passphrase (struct crypto_s *crypto, const char *key,
				  size_t len);
gpg_error_t set_pinentry_mode (struct agent_s *agent, const char *mode);
gpg_error_t get_pubkey (struct crypto_s *crypto, const char *hexgrip,
			gcry_sexp_t * result);
gpg_error_t get_pubkey_bin (struct crypto_s *crypto,
			    const unsigned char *grip, gcry_sexp_t * result);
gpg_error_t agent_set_pinentry_options (struct agent_s *agent);
gpg_error_t agent_export_common (struct crypto_s *crypto, const char *keygrip,
				 const char *sign_keygrip, int no_passphrase,
				 const void *data, size_t datalen,
				 const char *outfile, const char *keyparams,
				 const char *keyfile);
char *default_key_params (struct crypto_s *crypto);
gpg_error_t agent_loopback_cb (void *user, const char *keyword);
gpg_error_t agent_passwd (struct crypto_s *crypto);
gpg_error_t kill_scd (struct agent_s *);
gpg_error_t agent_extract_key (struct crypto_s *crypto, unsigned char **result,
			       size_t * result_len);
gpg_error_t agent_verify (gcry_sexp_t pkey, gcry_sexp_t sig_sexp,
			  const void *data, size_t len);
gpg_error_t send_agent_common_options (struct agent_s *agent);

#endif
