/*
    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    Ben Kibbey <bjk@luxsci.net>

    This file is part of pwmd.

    Pwmd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Pwmd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Pwmd.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CACHE_H
#define CACHE_H

#include <pthread.h>
#include <gpg-error.h>
#include <time.h>

struct cache_data_s
{
  void *pubkey;			/* needed for SAVE */
  void *sigkey;			/* needed for SAVE */
  void *doc;			/* AES-128 encrypted */
  size_t doclen;
  void *key;                    /* When use_agent == 0, hashed. */
  size_t keylen;
  unsigned char *crc;		/* Checksum of the data file. */
};

typedef struct
{
  unsigned char filename[16];	/* MD5 hash. */
  unsigned char grip[41];	/* key grip for this file */
  int timeout;			/* cache_timer_thread(). */
  int reset;			/* To reset .timeout. */
  int defer_clear;		/* Another thread wants to clear the cache
				   entry for this file. Prevent resetting the
				   timer until this flag is cleared. */
  pthread_mutex_t *mutex;	/* Data file mutex. */
  struct cache_data_s *data;
  unsigned refcount;
} file_cache_t;

unsigned char *cache_iv;
unsigned char *cache_key;
size_t cache_blocksize;
size_t cache_keysize;

void cache_adjust_timeout ();
gpg_error_t cache_set_timeout (const unsigned char *md5filename, int timeout);
gpg_error_t cache_iscached (const char *filename, int *defer);
gpg_error_t cache_clear (const unsigned char *md5filename);
gpg_error_t cache_add_file (const unsigned char *md5file,
                            const unsigned char *grip, struct cache_data_s *,
                            int timeout);
void cache_deinit (int atfork);
gpg_error_t cache_init ();
void cache_mutex_init ();
unsigned cache_file_count ();
struct cache_data_s *cache_get_data (const unsigned char *md5file);
struct cache_data_s *cache_get_data_filename (const char *file);
gpg_error_t cache_set_data (const unsigned char *md5file,
			    struct cache_data_s *, const unsigned char *grip);
gpg_error_t cache_is_shadowed (const char *grip);
gpg_error_t cache_lock_mutex (void *ctx, const unsigned char *md5file,
			      long mutex_timeout, int add, int timeout);
gpg_error_t cache_unlock_mutex (const unsigned char *md5file, int remove);
void cache_lock ();
void cache_unlock ();
void free_cache_data_once (struct cache_data_s *data);
gpg_error_t cache_defer_clear (const unsigned char *md5file);

#endif
