/*
    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    Ben Kibbey <bjk@luxsci.net>

    This file is part of pwmd.

    Pwmd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Pwmd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Pwmd.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gcrypt.h>
#include <string.h>

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#include "cipher.h"

unsigned
gcrypt_to_cipher (int n)
{
  if (n == GCRY_CIPHER_AES128)
    return PWMD_CIPHER_AES128;
  else if (n == GCRY_CIPHER_AES192)
    return PWMD_CIPHER_AES192;
  else if (n == GCRY_CIPHER_AES256)
    return PWMD_CIPHER_AES256;
  else if (n == GCRY_CIPHER_SERPENT128)
    return PWMD_CIPHER_SERPENT128;
  else if (n == GCRY_CIPHER_SERPENT192)
    return PWMD_CIPHER_SERPENT192;
  else if (n == GCRY_CIPHER_SERPENT256)
    return PWMD_CIPHER_SERPENT256;
  else if (n == GCRY_CIPHER_CAMELLIA128)
    return PWMD_CIPHER_CAMELLIA128;
  else if (n == GCRY_CIPHER_CAMELLIA192)
    return PWMD_CIPHER_CAMELLIA192;
  else if (n == GCRY_CIPHER_CAMELLIA256)
    return PWMD_CIPHER_CAMELLIA256;
  else if (n == GCRY_CIPHER_BLOWFISH)
    return PWMD_CIPHER_BLOWFISH;
  else if (n == GCRY_CIPHER_3DES)
    return PWMD_CIPHER_3DES;
  else if (n == GCRY_CIPHER_CAST5)
    return PWMD_CIPHER_CAST5;
  else if (n == GCRY_CIPHER_TWOFISH)
    return PWMD_CIPHER_TWOFISH;
  else if (n == GCRY_CIPHER_TWOFISH128)
    return PWMD_CIPHER_TWOFISH128;

  return PWMD_CIPHER_AES256;
}

int
cipher_to_gcrypt (int flags)
{
  if (flags < 0)
    return flags;

  if (flags & PWMD_CIPHER_AES128)
    return GCRY_CIPHER_AES128;
  else if (flags & PWMD_CIPHER_AES192)
    return GCRY_CIPHER_AES192;
  else if (flags & PWMD_CIPHER_AES256)
    return GCRY_CIPHER_AES256;
  else if (flags & PWMD_CIPHER_SERPENT128)
    return GCRY_CIPHER_SERPENT128;
  else if (flags & PWMD_CIPHER_SERPENT192)
    return GCRY_CIPHER_SERPENT192;
  else if (flags & PWMD_CIPHER_SERPENT256)
    return GCRY_CIPHER_SERPENT256;
  else if (flags & PWMD_CIPHER_CAMELLIA128)
    return GCRY_CIPHER_CAMELLIA128;
  else if (flags & PWMD_CIPHER_CAMELLIA192)
    return GCRY_CIPHER_CAMELLIA192;
  else if (flags & PWMD_CIPHER_CAMELLIA256)
    return GCRY_CIPHER_CAMELLIA256;
  else if (flags & PWMD_CIPHER_BLOWFISH)
    return GCRY_CIPHER_BLOWFISH;
  else if (flags & PWMD_CIPHER_3DES)
    return GCRY_CIPHER_3DES;
  else if (flags & PWMD_CIPHER_CAST5)
    return GCRY_CIPHER_CAST5;
  else if (flags & PWMD_CIPHER_TWOFISH)
    return GCRY_CIPHER_TWOFISH;
  else if (flags & PWMD_CIPHER_TWOFISH128)
    return GCRY_CIPHER_TWOFISH128;

  return -1;
}

int
cipher_string_to_cipher (const char *str)
{
  int flags = 0;

  if (!strcasecmp (str, "aes128"))
    flags = PWMD_CIPHER_AES128;
  else if (!strcasecmp (str, "aes192"))
    flags = PWMD_CIPHER_AES192;
  else if (!strcasecmp (str, "aes256"))
    flags = PWMD_CIPHER_AES256;
  else if (!strcasecmp (str, "serpent128"))
    flags = PWMD_CIPHER_SERPENT128;
  else if (!strcasecmp (str, "serpent192"))
    flags = PWMD_CIPHER_SERPENT192;
  else if (!strcasecmp (str, "serpent256"))
    flags = PWMD_CIPHER_SERPENT256;
  else if (!strcasecmp (str, "camellia128"))
    flags = PWMD_CIPHER_CAMELLIA128;
  else if (!strcasecmp (str, "camellia192"))
    flags = PWMD_CIPHER_CAMELLIA192;
  else if (!strcasecmp (str, "camellia256"))
    flags = PWMD_CIPHER_CAMELLIA256;
  else if (!strcasecmp (str, "blowfish"))
    flags = PWMD_CIPHER_BLOWFISH;
  else if (!strcasecmp (str, "cast5"))
    flags = PWMD_CIPHER_CAST5;
  else if (!strcasecmp (str, "3des"))
    flags = PWMD_CIPHER_3DES;
  else if (!strcasecmp (str, "twofish256"))
    flags = PWMD_CIPHER_TWOFISH;
  else if (!strcasecmp (str, "twofish128"))
    flags = PWMD_CIPHER_TWOFISH128;
  else
    return -1;

  return flags;
}

int
cipher_string_to_gcrypt (const char *str)
{
  return cipher_to_gcrypt (cipher_string_to_cipher (str));
}

uint64_t
set_cipher_flag (uint64_t flags, int algo)
{
  // Clear all previous cipher bits but keep the flags.
  flags &= ~((uint16_t) ((uint32_t) (flags & 0xFFFFFFFF)));
  flags |= gcrypt_to_cipher (algo);
  return flags;
}
