/*
    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    Ben Kibbey <bjk@luxsci.net>

    This file is part of pwmd.

    Pwmd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Pwmd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Pwmd.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef COMMON_H
#define COMMON_H

#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include "pwmd-error.h"
#include <gcrypt.h>
#include <assuan.h>
#include <stdint.h>
#include <libxml/xmlerror.h>
#include <pwd.h>

#ifdef ENABLE_NLS
#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif
#endif

#ifdef HAVE_LIMITS_H
#include <limits.h>
#ifndef LINE_MAX
#ifdef _POSIX2_LINE_MAX
#define LINE_MAX _POSIX2_LINE_MAX
#else
#define LINE_MAX 2048
#endif
#endif
#endif

#ifndef _
#include "gettext.h"
#define _(msgid) gettext(msgid)
#endif

#ifdef WITH_GNUTLS
#include "tls.h"
#endif
#include "status.h"
#include "agent.h"

#define CLIENT_STATE_UNKNOWN	0
#define CLIENT_STATE_INIT	1
#define CLIENT_STATE_IDLE	2
#define CLIENT_STATE_COMMAND	3
#define CLIENT_STATE_DISCON	4

struct client_thread_s
{
  pthread_t tid;
  char *name;
  int fd;
  pthread_mutex_t status_mutex;
  struct status_msg_s *msg_queue;
  int status_msg_pipe[2];
  int wrote_status;
  struct client_s *cl;
  int atfork;
  assuan_peercred_t peer;
  unsigned state;
#ifdef WITH_GNUTLS
  int timeout;
  int buffer_timeout;
  int last_buffer_size;
  int remote;
  struct tls_s *tls;
  char *peeraddr;
#endif
};

struct client_s
{
  assuan_context_t ctx;
  int flock_fd;
  void *doc;			/* xmlDocPtr */
  xmlErrorPtr xml_error;
  char *filename;
  unsigned char md5file[16];
  struct client_thread_s *thd;
  struct crypto_s *crypto;
  uint32_t opts;
  uint32_t flags;
  char *import_root;
  long lock_timeout;		/* In tenths of a second. */
  gpg_error_t last_rc;
  char *last_error;		/* ELOOP element path. */
  unsigned char *crc;		/* Of the data file. */
  struct pinentry_option_s pinentry_opts;
};

pthread_key_t thread_name_key;
pthread_mutex_t cn_mutex;
struct slist_s *cn_thread_list;
char **debug_level;

int assuan_log_cb (assuan_context_t ctx, void *data, unsigned cat,
		   const char *msg);
void log_write (const char *fmt, ...);
gpg_error_t send_error (assuan_context_t ctx, gpg_error_t e);
gpg_error_t do_validate_peer (assuan_context_t ctx, const char *section,
			      assuan_peercred_t * peer);
gpg_error_t acl_check_common (struct client_s *client, const char *user,
                              uid_t uid, gid_t gid, int *allowed);
gpg_error_t peer_is_invoker(struct client_s *client);
void update_client_state (struct client_s *client, unsigned s);
gpg_error_t lock_flock (assuan_context_t ctx, const char *filename, int type,
                        int *fd);
void unlock_flock (int *fd);

#endif
