/*
    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    Ben Kibbey <bjk@luxsci.net>

    This file is part of pwmd.

    Pwmd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Pwmd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Pwmd.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PINENTRY_H
#define PINENTRY_H

typedef enum
{
  PINENTRY_OPEN,
  PINENTRY_SAVE,
  PINENTRY_SAVE_CONFIRM,
  PINENTRY_FAIL_CONFIRM
} pinentry_cmd_t;

struct pinentry_option_s
{
  char *title;
  char *desc;
  char *prompt;
  char *ttyname;
  char *ttytype;
  char *display;
  char *lc_ctype;
  char *lc_messages;
  int timeout;
};

struct pinentry_s
{
  char *filename;
  assuan_context_t ctx;
  struct pinentry_option_s pinentry_opts;
  int which;
  char *name;
  int has_lock;
  int enable;
  membuf_t data;
  gpg_error_t (*inquire_cb) (void *data, const char *line);
  void *inquire_data;
};

pthread_mutex_t pin_mutex;
struct client_s;

gpg_error_t pinentry_fork (assuan_context_t ctx);
void cleanup_pinentry (struct pinentry_s *pin);
gpg_error_t pinentry_getpin (struct pinentry_s *pin, char **result,
			     pinentry_cmd_t which);
void pinentry_unlock (struct pinentry_s *pin);
gpg_error_t pinentry_lock (assuan_context_t ctx, struct pinentry_s *pin);
void *pinentry_timeout_thread (void *arg);
gpg_error_t pinentry_iterate (struct client_s *cl, int read_ready);
struct pinentry_s *pinentry_init (const char *filename);
void pinentry_deinit (struct pinentry_s *);
void pinentry_set_defaults (struct pinentry_option_s *opts);
void reset_pin_defaults (struct pinentry_s *pin);
void pinentry_free_opts (struct pinentry_option_s *opts);
gpg_error_t pinentry_merge_options (struct pinentry_option_s *dst,
				    struct pinentry_option_s *src);

#endif
