/*
    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    Ben Kibbey <bjk@luxsci.net>

    This file is part of pwmd.

    Pwmd is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    Pwmd is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Pwmd.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef RCFILE_H
#define RCFILE_H

#include <pthread.h>
#include <sys/types.h>
#include "util-slist.h"

/* This is a string so ealier versions of glib can be used. */
#define DEFAULT_ITERATION_PROGRESS	"2000"

#define INVALID_PRIORITY                "-99"

struct config_section_s
{
  char *name;
  struct slist_s *params;
};

#define INVOKING_UID	0
#define INVOKING_GID	1
#define INVOKING_TLS	2

struct invoking_user_s
{
  int type;
  int not;
  union {
      uid_t uid;
      char *id;
  };
  struct invoking_user_s *next;
};

#ifdef HAVE_MLOCKALL
int disable_mlock;
#endif
char *logfile;
char *rcfile;
char *homedir;
int max_recursion_depth;
int disable_list_and_dump;
struct slist_s *global_config;
int log_syslog;
pthread_mutex_t rcfile_mutex;
pthread_cond_t rcfile_cond;
pthread_t rcfile_tid;
struct invoking_user_s *invoking_users;
int log_keepopen;

void free_invoking_users (struct invoking_user_s *);
struct slist_s *config_parse (const char *filename, int reload);
void config_clear_keys ();
void config_free (struct slist_s *config);

char *config_get_value (const char *section, const char *what);
long config_get_long (const char *section, const char *what);
int config_get_boolean (const char *section, const char *what);
int config_get_integer (const char *section, const char *what);
char *config_get_string (const char *section, const char *what);
char **config_get_list (const char *section, const char *what);
unsigned long long config_get_ulonglong (const char *section, const char *what);

int config_set_list_param (struct slist_s **config, const char *section,
			   const char *name, const char *value);
char **config_get_list_param (struct slist_s *config, const char *section,
			      const char *what, int *exists);
int config_set_int_param (struct slist_s **config, const char *section,
			  const char *name, const char *value);
int config_get_int_param (struct slist_s *config, const char *section,
			  const char *name, int *exists);
int config_get_bool_param (struct slist_s *config, const char *section,
			   const char *name, int *exists);
int config_set_bool_param (struct slist_s **config, const char *section,
			   const char *name, const char *value);
struct slist_s *config_keep_save ();
void config_keep_restore (struct slist_s *);

#endif
